#include "stdint.h"
#include "MDMA.h"
#include <sys/platform.h>

#pragma align 32
#pragma section("seg_l1_block1")
uint32_t ucIntBuff1[MAX_DMA_SIZE_BYTES];

#pragma align 32
#pragma section("seg_l1_block2")
uint32_t ucIntBuff2[MAX_DMA_SIZE_BYTES];

float MDMA_Throughput_Measure(MDMA_PARAM_LIST MDMAParam)
{
	uint32_t uiClockStart;
	uint32_t uiClockStop;
	uint32_t uiCycles;
	float fThroughput;
	uint32_t  psrc;
	uint32_t  pdst;
	uint32_t uiSrcDMACfg=0;
	uint32_t uiDstDMACfg=BITM_DMA_CFG_WNR|ENUM_DMA_CFG_XCNT_INT;

	/*Configuring MDMAs as secure masters*/
	*pREG_SPU0_SECUREP102=0x3;
	*pREG_SPU0_SECUREP103=0x3;
	*pREG_SPU0_SECUREP104=0x3;
	*pREG_SPU0_SECUREP105=0x3;
	*pREG_SPU0_SECUREP92=0x3;
	*pREG_SPU0_SECUREP93=0x3;

	uiClockStart = 0x7FFFFFFF;

	psrc=GetMemPointer(MDMAParam.uiSrcMem);
	pdst=GetMemPointer(MDMAParam.uiDestMem);

	uiSrcDMACfg|=GetMSIZEConfig(MDMAParam.uiSrcMSIZE);
	uiDstDMACfg|=GetMSIZEConfig(MDMAParam.uiDstMSIZE);

	if(MDMAParam.uiMDMAStreamNo==0)
	{
		uiSrcDMACfg|=ENUM_DMA_CFG_PSIZE04;
		uiDstDMACfg|=ENUM_DMA_CFG_PSIZE04;
		*pREG_DMA8_BWMCNT=0x7FFFFFFF;
		MDMA_CONFIG(8, uiSrcDMACfg, psrc, MDMAParam.uiSrcMSIZE, MDMAParam.uiBytes/MDMAParam.uiSrcMSIZE,9,uiDstDMACfg, pdst, MDMAParam.uiDstMSIZE, MDMAParam.uiBytes/MDMAParam.uiDstMSIZE);
		WAIT_FOR_DMADONE(9);
		uiClockStop = *pREG_DMA8_BWMCNT_CUR;
	}
	if(MDMAParam.uiMDMAStreamNo==1)
	{
		uiSrcDMACfg|=ENUM_DMA_CFG_PSIZE04;
		uiDstDMACfg|=ENUM_DMA_CFG_PSIZE04;
		*pREG_DMA18_BWMCNT=0x7FFFFFFF;
		MDMA_CONFIG(18, uiSrcDMACfg, psrc, MDMAParam.uiSrcMSIZE, MDMAParam.uiBytes/MDMAParam.uiSrcMSIZE,19,uiDstDMACfg, pdst, MDMAParam.uiDstMSIZE, MDMAParam.uiBytes/MDMAParam.uiDstMSIZE);
		WAIT_FOR_DMADONE(19);
		uiClockStop = *pREG_DMA18_BWMCNT_CUR;
	}
	if(MDMAParam.uiMDMAStreamNo==2)
	{
		uiSrcDMACfg|=ENUM_DMA_CFG_PSIZE04;
		uiDstDMACfg|=ENUM_DMA_CFG_PSIZE04;
		*pREG_DMA39_BWMCNT=0x7FFFFFFF;
		MDMA_CONFIG(39, uiSrcDMACfg, psrc, MDMAParam.uiSrcMSIZE, MDMAParam.uiBytes/MDMAParam.uiSrcMSIZE,40,uiDstDMACfg, pdst, MDMAParam.uiDstMSIZE, MDMAParam.uiBytes/MDMAParam.uiDstMSIZE);
		WAIT_FOR_DMADONE(40);
		uiClockStop = *pREG_DMA39_BWMCNT_CUR;
	}
	if(MDMAParam.uiMDMAStreamNo==3)
	{
		uiSrcDMACfg|=ENUM_DMA_CFG_PSIZE08;
		uiDstDMACfg|=ENUM_DMA_CFG_PSIZE08;
		*pREG_DMA43_BWMCNT=0x7FFFFFFF;
		MDMA_CONFIG(43, uiSrcDMACfg, psrc, MDMAParam.uiSrcMSIZE, MDMAParam.uiBytes/MDMAParam.uiSrcMSIZE,44,uiDstDMACfg, pdst, MDMAParam.uiDstMSIZE, MDMAParam.uiBytes/MDMAParam.uiDstMSIZE);
		WAIT_FOR_DMADONE(44);
		uiClockStop = *pREG_DMA43_BWMCNT_CUR;
	}

	uiCycles=uiClockStart-uiClockStop;

	fThroughput=((float)(MDMAParam.uiBytes)*MDMAParam.fMDMAClockSpeedMHz)/((float)uiCycles);

	return fThroughput;
}

uint32_t GetMemPointer(uint32_t uiMemType)
{
	if(uiMemType==MEMTYPE_L1S1)
		return (uint32_t)(&ucIntBuff1)|0x28000000;
	else if(uiMemType==MEMTYPE_L1S2)
		return (uint32_t)(&ucIntBuff2)|0x28400000;
	else if(uiMemType==MEMTYPE_L1S1_CORE2)
				return (uint32_t)(&ucIntBuff2)|0x28800000;
	else if(uiMemType==MEMTYPE_L2)
			return L2_START_ADDRESS;
	else if(uiMemType==MEMTYPE_L3_DMC0)
			return DMC0_START_ADDRESS;
	else if(uiMemType==MEMTYPE_L3_DMC1)
			return DMC1_START_ADDRESS;
	else
		return (uint32_t)(&ucIntBuff1)|0x28000000;
}

uint32_t GetMSIZEConfig(uint32_t uiMSIZE)
{
	uint32_t MSIZEConfig;

	if(uiMSIZE==1)
		return ENUM_DMA_CFG_MSIZE01;
	if(uiMSIZE==2)
		return ENUM_DMA_CFG_MSIZE02;

	if(uiMSIZE==4)
		return ENUM_DMA_CFG_MSIZE04;
	if(uiMSIZE==8)
		return ENUM_DMA_CFG_MSIZE08;
	if(uiMSIZE==16)
		return ENUM_DMA_CFG_MSIZE16;
	if(uiMSIZE==32)
		return ENUM_DMA_CFG_MSIZE32;

	return MSIZEConfig;
}
