/*This code shows how a 4 byte aligned MDMA transfer can be optimized by splitting it into
 * three descriptor based MDMA transfers with the middle one configured for MSIZE=32 bytes.
  */

#include "main.h"
#include <sys/platform.h>
#include <sys/adi_core.h>
#include "adi_initialize.h"
#include "stdint.h"
#include "time.h"
#include "stdio.h"
/** 
 *
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

#define MDMA_START_ADDRESS 0x80000004
//#define MDMA_START_ADDRESS 0x80000000
//#define MDMA_START_ADDRESS 0x80000008
#define DMA_COUNT_BYTES 20000
#define TCB_SIZE 4

#define SRC_DMA_CFG	 	ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_PSIZE08
#define DST_DMA_CFG		BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_XCNT_INT


#define SRC_DMA_CFG1	 	(ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define SRC_DMA_CFG2		(ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define SRC_DMA_CFG3	    (ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_PSIZE08)
#define SRC_DMA_CFG4	    (ENUM_DMA_CFG_EN|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_PSIZE08)

#define DST_DMA_CFG1 	 	(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define DST_DMA_CFG2 	 	(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_DSCARRAY|ENUM_DMA_CFG_FETCH04)
#define DST_DMA_CFG3 		(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE04|ENUM_DMA_CFG_XCNT_INT)
#define DST_DMA_CFG4 		(ENUM_DMA_CFG_EN|BITM_DMA_CFG_WNR|ENUM_DMA_CFG_PSIZE08|ENUM_DMA_CFG_MSIZE32|ENUM_DMA_CFG_XCNT_INT)

#define SRC_DMA_MSIZE1 4
#define DST_DMA_MSIZE1 4

#define SRC_DMA_MSIZE2 32
#define DST_DMA_MSIZE2 32

#define SRC_DMA_MSIZE3 4
#define DST_DMA_MSIZE3 4

#define DMA_COUNT1		7
#define DMA_COUNT2		30
#define DMA_COUNT3		3

#pragma align 32
unsigned char dstbuff[DMA_COUNT_BYTES+4];

//This macro can be used to configure an MDMA stream by providing the source and destination DMA channel details
#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG |= 0x1;\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG |= 0x1;

#define DISABLE_MDMA(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = 0;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = 0;

//Macro to wait for DMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));


uint32_t uiSrcTCB1[3*TCB_SIZE];
uint32_t uiDstTCB1[3*TCB_SIZE];

volatile unsigned int clock_stop;
volatile unsigned int clock_start;
volatile unsigned int cycles;

volatile int ii;
#define START_CYCLE_COUNT\
	clock_start = clock();

#define STOP_CYCLE_COUNT\
	clock_stop = clock();\
	cycles = clock_stop - clock_start;

void Mem_Copy_MDMA(unsigned int srcadd, unsigned int dstadd, unsigned int count);
void Mem_Copy_MDMA_MSIZE4(unsigned int srcadd, unsigned int dstadd, unsigned int count);
int Buff_Init(unsigned int* buff, unsigned int count);
int Buff_Compare(unsigned int* buff1, unsigned int* buff2, unsigned int count);

int main(int argc, char *argv[])
{
	int i; int failure=0;

	*pREG_SPU0_SECUREP93=0x3;

	DISABLE_MDMA(43, 44);

	Buff_Init((unsigned int*)MDMA_START_ADDRESS, DMA_COUNT_BYTES/4);

	//Old MDMA copy method - mix of MSIZE 32 and MSIZE 4 bytes
	START_CYCLE_COUNT;
	Mem_Copy_MDMA_MSIZE4(MDMA_START_ADDRESS, (unsigned int)dstbuff|0x28000000,DMA_COUNT_BYTES/4);
	STOP_CYCLE_COUNT;

	DISABLE_MDMA(43, 44);


	failure=Buff_Compare((unsigned int*)MDMA_START_ADDRESS, (unsigned int*)dstbuff, DMA_COUNT_BYTES/4);
	if(failure==0)
		printf("\n\nOld MDMA copy method passed as expected....cycles taken to transfer %d bytes = %d ", DMA_COUNT_BYTES, cycles);
	else
		printf("\n\nOld MDMA copy failed....\n\n");

	//Clear the destination buffer
	for(i=0;i<DMA_COUNT_BYTES/4;i++)
	{
		dstbuff[i]=0;
	}


	//New MDMA copy method - mix of MSIZE 32 and MSIZE 4 bytes
	START_CYCLE_COUNT;
	Mem_Copy_MDMA(MDMA_START_ADDRESS, (unsigned int)dstbuff|0x28000000,DMA_COUNT_BYTES/4);
	STOP_CYCLE_COUNT;

	failure=Buff_Compare((unsigned int*)MDMA_START_ADDRESS, (unsigned int*)dstbuff, DMA_COUNT_BYTES/4);
	if(failure==0)
		printf("\n\nNew MDMA copy method passed as expected....cycles taken to transfer %d bytes = %d", DMA_COUNT_BYTES, cycles);
	else
		printf("\n\nNew MDMA copy failed....\n\n");

	return 0;
}

void Mem_Copy_MDMA(unsigned int srcadd, unsigned int dstadd, unsigned int count)
{
	unsigned int SrcDMACount1, SrcDMACount2, SrcDMACount3;
	unsigned int DstDMACount1, DstDMACount2, DstDMACount3;
	unsigned int TotalByteCountDst=count*4;
	unsigned int TotalByteCountSrc=count*4;

	//Source count should be multiple of 8 for PSIZE 32 bytes
	if(TotalByteCountSrc%8!=0)
	{
		TotalByteCountSrc+=4;
	}

	//Calculate the TCB values for source DMA channel
	if((TotalByteCountSrc/32)==0) 		//If total number of bytes are less than 32 bytes - need only one descriptor of MSIZE=4 bytes
	{
		uiSrcTCB1[0]=srcadd;
		uiSrcTCB1[1]=SRC_DMA_CFG3;
		uiSrcTCB1[2]=TotalByteCountSrc/4;
		uiSrcTCB1[3]=4;
	}

	else if((srcadd&0x0000001F)==0)							//If the start address is 32 byte aligned
	{
		SrcDMACount1=(TotalByteCountSrc/32)*32;
		SrcDMACount2=TotalByteCountSrc-SrcDMACount1;

		uiSrcTCB1[0]=srcadd;
		uiSrcTCB1[2]=SrcDMACount1/32;
		uiSrcTCB1[3]=32;

		if(SrcDMACount2==0)
		{
			uiSrcTCB1[1]=SRC_DMA_CFG4;
		}
		else
		{
			uiSrcTCB1[1]=SRC_DMA_CFG2;
			uiSrcTCB1[4]=(srcadd+SrcDMACount1);
			uiSrcTCB1[5]=SRC_DMA_CFG3;
			uiSrcTCB1[6]=SrcDMACount2/4;
			uiSrcTCB1[7]=4;
		}
	}
	else
	{
		SrcDMACount1=32-(srcadd&0x0000001F);
		SrcDMACount2=TotalByteCountSrc-SrcDMACount1;

		if(SrcDMACount2<32)
		{
			SrcDMACount3=0;
		}
		else
		{
			SrcDMACount3=SrcDMACount2%32;
			SrcDMACount2=(SrcDMACount2/32)*32;
		}

		if(SrcDMACount3==0)
		{
			uiSrcTCB1[0]=srcadd;
			uiSrcTCB1[1]=SRC_DMA_CFG4;
			uiSrcTCB1[2]=TotalByteCountSrc/4;
			uiSrcTCB1[3]=4;
		}
		else
		{
			uiSrcTCB1[0]=srcadd;
			uiSrcTCB1[1]=SRC_DMA_CFG1;
			uiSrcTCB1[2]=SrcDMACount1/4;
			uiSrcTCB1[3]=4;

			uiSrcTCB1[4]=(srcadd+SrcDMACount1);
			uiSrcTCB1[5]=SRC_DMA_CFG2;
			uiSrcTCB1[6]=SrcDMACount2/32;
			uiSrcTCB1[7]=32;

			uiSrcTCB1[8]=(srcadd+SrcDMACount1+SrcDMACount2);
			uiSrcTCB1[9]=SRC_DMA_CFG3;
			uiSrcTCB1[10]=SrcDMACount3/4;
			uiSrcTCB1[11]=4;
		}
	}

	//Calculate the TCB values for destination DMA channel
	if((TotalByteCountDst/32)==0) 						//If total number of bytes are less than 32 bytes - need only one descriptor of MSIZE=4 bytes
	{
		uiDstTCB1[0]=dstadd;
		uiDstTCB1[1]=DST_DMA_CFG3;
		uiDstTCB1[2]=TotalByteCountDst/4;
		uiDstTCB1[3]=4;
	}

	else if((dstadd&0x0000001F)==0)							//If the start address is 32 byte aligned
	{
		DstDMACount1=(TotalByteCountDst/32)*32;
		DstDMACount2=TotalByteCountDst-DstDMACount1;

		uiDstTCB1[0]=dstadd;
		uiDstTCB1[2]=DstDMACount1/32;
		uiDstTCB1[3]=32;

		if(DstDMACount2==0)
		{
			uiDstTCB1[1]=DST_DMA_CFG4;
		}
		else
		{
			uiDstTCB1[1]=DST_DMA_CFG2;
			uiDstTCB1[4]=(dstadd+DstDMACount1);
			uiDstTCB1[5]=DST_DMA_CFG3;
			uiDstTCB1[6]=DstDMACount2/4;
			uiDstTCB1[7]=4;
		}
	}
	else
	{
		DstDMACount1=32-(dstadd&0x0000001F);
		DstDMACount2=TotalByteCountDst-DstDMACount1;

		if(DstDMACount2<32)
		{
			DstDMACount3=0;
		}
		else
		{
			DstDMACount3=DstDMACount2%32;
			DstDMACount2=(DstDMACount2/32)*32;
		}

		if(DstDMACount3==0)
		{
			uiDstTCB1[0]=dstadd;
			uiDstTCB1[1]=DST_DMA_CFG4;
			uiDstTCB1[2]=TotalByteCountDst/4;
			uiDstTCB1[3]=4;
		}
		else
		{
			uiDstTCB1[0]=dstadd;
			uiDstTCB1[1]=DST_DMA_CFG1;
			uiDstTCB1[2]=DstDMACount1/4;
			uiDstTCB1[3]=4;

			uiDstTCB1[4]=(dstadd+DstDMACount1);
			uiDstTCB1[5]=DST_DMA_CFG2;
			uiDstTCB1[6]=DstDMACount2/32;
			uiDstTCB1[7]=32;

			uiDstTCB1[8]=(dstadd+DstDMACount1+DstDMACount2);
			uiDstTCB1[9]=DST_DMA_CFG3;
			uiDstTCB1[10]=DstDMACount3/4;
			uiDstTCB1[11]=4;
		}

	}

	*pREG_DMA43_DSCPTR_CUR=(uint32_t)uiSrcTCB1|0x28000000;
	*pREG_DMA44_DSCPTR_CUR=(uint32_t)uiDstTCB1|0x28000000;
	*pREG_DMA44_CFG=SRC_DMA_CFG1;
	*pREG_DMA43_CFG=SRC_DMA_CFG1;

	WAIT_FOR_DMADONE(44);
}

void Mem_Copy_MDMA_MSIZE4(unsigned int srcadd, unsigned int dstadd, unsigned int count)
{
	unsigned int srccount=count;

	if(srccount%8!=0)
	{
		srccount+=4;
	}

	MDMA_CONFIG(43,SRC_DMA_CFG, MDMA_START_ADDRESS, SRC_DMA_MSIZE1, srccount, 44,DST_DMA_CFG, dstadd, DST_DMA_MSIZE1, count);
	WAIT_FOR_DMADONE(44);
}

int Buff_Compare(unsigned int* buff1, unsigned int* buff2, unsigned int count)
{
	int i; int failures=0;

	for(i=0;i<count;i++)
	{
		if(*buff1!=*buff2)
			failures++;

		buff1++; buff2++;
	}

	return failures;
}

int Buff_Init(unsigned int* buff, unsigned int count)
{
	int i;

	for(i=0;i<count;i++)
	{
		*buff=rand();
		buff++;
	}
	return 1;
}
