#include "DMCInit.h"

#pragma optimize_off
void DMCInit(DMC_PARAM DMC_Param_List)
{
	//PHY Initialization
	if(DMC_Param_List.phy_init_required==true)
		DMC_PHY_Init(DMC_Param_List);

	//Controller Initialization
	DMC_Controller_Init(DMC_Param_List);

	*pREG_DMC0_EFFCTL = 0xF80000;
}

#pragma optimize_off
void DMC_PHY_Init(DMC_PARAM DMC_Param_List)
{
	int i;
	if(DMC_Param_List.dmc_no==0)
	{
	   //1. Set DDR mode to DDR3/DDR2/LPDDR in DMCx_PHY_CTL4 register
		if(DMC_Param_List.ddr_mode==DDR3_MODE)
		{
			*pREG_DMC0_PHY_CTL4 = ENUM_DMC_PHY_CTL4_DDR3;
		}
		else if(DMC_Param_List.ddr_mode==DDR2_MODE)
		{
			*pREG_DMC0_PHY_CTL4 = ENUM_DMC_PHY_CTL4_DDR2;
		}
		else if(DMC_Param_List.ddr_mode==LPDDR_MODE)
		{
			*pREG_DMC0_PHY_CTL4 = ENUM_DMC_PHY_CTL4_LPDDR;
		}

	   //2.	Make sure that the bits 6, 7, 25, and 27 of the DMC_PHY_CTL3 register are set
	   *pREG_DMC0_PHY_CTL3=0xA0000C0;

	   //3. For DDR2/DDR3 mode, make sure that the bits 0, 1, 2, 3 of the DMC_PHY_CTL0 register and the bits 26, 27, 28, 29, 30, 31 of the DMC_PHY_CTL2 are set.
		if(DMC_Param_List.ddr_mode==DDR3_MODE||DMC_Param_List.ddr_mode==DDR2_MODE)
		{
			*pREG_DMC0_PHY_CTL2= 0xFC000000;
			*pREG_DMC0_PHY_CTL0= 0x0000000F;
		}

		//4.For DDR3 mode, set bit 1 and configure bits [5:2] of the DMC_CPHY_CTL register with WL=CWL+AL in DCLK cycles.
		if(DMC_Param_List.ddr_mode==DDR3_MODE)
		{
			*pREG_DMC0_CPHY_CTL=DMC_Param_List.dmc_cphyctl_value;
		}

	    //5.Perform On Die Termination (ODT) and Driver Impedance Calibration
		if(DMC_Param_List.ddr_mode==LPDDR_MODE)
		{
			//Bypass processor ODT
			*pREG_DMC0_PHY_CTL1 = 0x80000;
		}
		else
		{
			//Configure ODT and drive impedance values in the DMCx_CAL_PADCTL2 register
			*pREG_DMC0_CAL_PADCTL2  =  DMC_Param_List.padctl2_value;

			//Make sure that the bits RTTCALEN, PDCALEN, PUCALEN of register DMCx_CAL_PADCTL0 are set (they are set at reset) and start calibration
			*pREG_DMC0_CAL_PADCTL0  =  BITM_DMC_CAL_PADCTL0_RTTCALEN|BITM_DMC_CAL_PADCTL0_PDCALEN|BITM_DMC_CAL_PADCTL0_PUCALEN|BITM_DMC_CAL_PADCTL0_CALSTRT;

			//Wait for PAD calibration to complete - 300 DCLK cycle. Worst case: CCLK=450 MHz, DCLK=125 MHz
			for(i=0;i<300*DMC_Param_List.cclk_dclk_ratio;i++);
		}
	}

	if(DMC_Param_List.dmc_no==1)
	{
	   //1. Set DDR mode to DDR3/DDR2/LPDDR in DMCx_PHY_CTL4 register
		if(DMC_Param_List.ddr_mode==DDR3_MODE)
		{
			*pREG_DMC1_PHY_CTL4 = ENUM_DMC_PHY_CTL4_DDR3;
		}
		else if(DMC_Param_List.ddr_mode==DDR2_MODE)
		{
			*pREG_DMC1_PHY_CTL4 = ENUM_DMC_PHY_CTL4_DDR2;
		}
		else if(DMC_Param_List.ddr_mode==LPDDR_MODE)
		{
			*pREG_DMC1_PHY_CTL4 = ENUM_DMC_PHY_CTL4_LPDDR;
		}

		//2.	Make sure that the bits 6, 7, 25, and 27 of the DMC_PHY_CTL3 register are set
		*pREG_DMC1_PHY_CTL3=0xA0000C0;

	    //3. For DDR2/DDR3 mode, make sure that the bits 0, 1, 2, 3 of the DMC_PHY_CTL0 register and the bits 26, 27, 28, 29, 30, 31 of the DMC_PHY_CTL2 are set.
		if(DMC_Param_List.ddr_mode==DDR3_MODE||DMC_Param_List.ddr_mode==DDR2_MODE)
		{
			*pREG_DMC1_PHY_CTL2= 0xFC000000;
			*pREG_DMC1_PHY_CTL0= 0x0000000F;
		}

		//4.For DDR3 mode, set bit 1 and configure bits [5:2] of the DMC_CPHY_CTL register with WL=CWL+AL in DCLK cycles.
		if(DMC_Param_List.ddr_mode==DDR3_MODE)
		{
			*pREG_DMC1_CPHY_CTL=DMC_Param_List.dmc_cphyctl_value;
		}

	   //5.Perform On Die Termination (ODT) and Driver Impedance Calibration
		if(DMC_Param_List.ddr_mode==LPDDR_MODE)
		{
			//Bypass processor ODT
			*pREG_DMC1_PHY_CTL1 = 0x80000;
		}
		else
		{
			//Configure ODT and drive impedance values in the DMCx_CAL_PADCTL2 register
			*pREG_DMC1_CAL_PADCTL2  =  DMC_Param_List.padctl2_value;

			//Make sure that the bits RTTCALEN, PDCALEN, PUCALEN of register DMCx_CAL_PADCTL0 are set (they are set at reset) and start calibration
			*pREG_DMC1_CAL_PADCTL0  =  BITM_DMC_CAL_PADCTL0_RTTCALEN|BITM_DMC_CAL_PADCTL0_PDCALEN|BITM_DMC_CAL_PADCTL0_PUCALEN|BITM_DMC_CAL_PADCTL0_CALSTRT;

			//Wait for PAD calibration to complete - 300 DCLK cycle. Worst case: CCLK=450 MHz, DCLK=125 MHz
			for(i=0;i<300*DMC_Param_List.cclk_dclk_ratio;i++);
		}
	}

}

#pragma optimize_off
void DMC_Controller_Init(DMC_PARAM DMC_Param_List)
{
	int temp;
	int* ptemp;

	if(DMC_Param_List.dmc_no==0)
	{
		//1. Program the DMC controller registers: DMCx_CFG, DMCx_TR0, DMCx_TR1, DMCx_TR2, DMCx_MR(DDR2/LPDDR)/DMCx_MR0(DDR3),
		//DMCx_EMR1(DDR2)/DMCx_MR1(DDR3), DMCx_EMR2(DDR2)/DMCx_EMR(LPDDR)/DMCx_MR2(DDR3)
		*pREG_DMC0_CFG = DMC_Param_List.dmc_cfg_value;
		*pREG_DMC0_TR0 = DMC_Param_List.dmc_tr0_value;
		*pREG_DMC0_TR1 = DMC_Param_List.dmc_tr1_value;
		*pREG_DMC0_TR2 = DMC_Param_List.dmc_tr2_value;
		*pREG_DMC0_MR =  DMC_Param_List.dmc_mr0_value;
		*pREG_DMC0_EMR1 = DMC_Param_List.dmc_mr1_value;
		*pREG_DMC0_EMR2 = DMC_Param_List.dmc_mr2_value;

		//2. Make sure that the DMC_DT_CALIB_ADDR register is programmed to an unused DMC location corresponding to a burst of 16 bytes (by default it is the starting address of the DMC address range).
		*pREG_DMC0_DT_CALIB_ADDR=DMC_Param_List.dmc_data_calib_add_value;

		//3. Program the DMCx_CTL register with INIT bit set to start the DMC initialization sequence
		*pREG_DMC0_CTL=DMC_Param_List.dmc_ctl_value;

		//4. Wait for the DMC initialization to complete by polling DMCx_STAT.INITDONE bit.
		while((*pREG_DMC0_STAT&BITM_DMC_STAT_INITDONE)==0);

		//5. Program the DMCx_CTL.DLLCTL register with 0x948 value (DATACYC=9,	DLLCALRDCNT=72).
		*pREG_DMC0_DLLCTL=0x00000948;

		//6. Workaround for anomaly#20000037
		if(DMC_Param_List.anomaly_20000037_applicable==true)
		{
			//Perform dummy read to any DMC location
			ptemp=0x80000000; temp=*ptemp;

			//Set DMCx_PHY_CTL0.RESETDAT bit
			*pREG_DMC0_PHY_CTL0|=0x1000;

			//Clear DMCx_PHY_CTL0.RESETDAT bit
			*pREG_DMC0_PHY_CTL0&=~0x1000;
		}
	}
	if(DMC_Param_List.dmc_no==1)
	{
		//1. Program the DMC controller registers: DMCx_CFG, DMCx_TR0, DMCx_TR1, DMCx_TR2, DMCx_MR(DDR2/LPDDR)/DMCx_MR0(DDR3),
		//DMCx_EMR1(DDR2)/DMCx_MR1(DDR3), DMCx_EMR2(DDR2)/DMCx_EMR(LPDDR)/DMCx_MR2(DDR3)
		*pREG_DMC1_CFG = DMC_Param_List.dmc_cfg_value;
		*pREG_DMC1_TR0 = DMC_Param_List.dmc_tr0_value;
		*pREG_DMC1_TR1 = DMC_Param_List.dmc_tr1_value;
		*pREG_DMC1_TR2 = DMC_Param_List.dmc_tr2_value;
		*pREG_DMC1_MR =  DMC_Param_List.dmc_mr0_value;
		*pREG_DMC1_EMR1 = DMC_Param_List.dmc_mr1_value;
		*pREG_DMC1_EMR2 = DMC_Param_List.dmc_mr2_value;

		//2. Make sure that the DMC_DT_CALIB_ADDR register is programmed to an unused DMC location corresponding to a burst of 16 bytes (by default it is the starting address of the DMC address range).
		*pREG_DMC1_DT_CALIB_ADDR=DMC_Param_List.dmc_data_calib_add_value;

		//3. Program the DMCx_CTL register with INIT bit set to start the DMC initialization sequence
		*pREG_DMC1_CTL=DMC_Param_List.dmc_ctl_value;

		//4. Wait for the DMC initialization to complete by polling DMCx_STAT.INITDONE bit.
		while((*pREG_DMC1_STAT&BITM_DMC_STAT_INITDONE)==0);

		//5. Program the DMCx_CTL.DLLCTL register with 0x948 value (DATACYC=9,	DLLCALRDCNT=72).
		*pREG_DMC1_DLLCTL=0x00000948;

		//7. Workaround for anomaly#20000037
		if(DMC_Param_List.anomaly_20000037_applicable==true)
		{
			//Perform dummy read to any DMC location
			ptemp=0x80000000; temp=*ptemp;

			//Set DMCx_PHY_CTL0.RESETDAT bit
			*pREG_DMC1_PHY_CTL0|=0x1000;

			//Clear DMCx_PHY_CTL0.RESETDAT bit
			*pREG_DMC1_PHY_CTL0&=~0x1000;
		}
	}
}

#pragma optimize_off
void DMC_Enter_Self_Refresh(uint32_t dmc_no)
{
	int temp; int* ptemp;

	if(dmc_no==0)
	{
		//Workaround for the anomaly#20000067
		*pREG_DMC0_PHY_CTL1|=0x40000;

		//Wait for the DMC to be in IDLE state
		while((*pREG_DMC0_STAT&BITM_DMC_STAT_IDLE)==0);

		//Set the SRREQ bit to enter self refresh mode
		*pREG_DMC0_CTL|= BITM_DMC_CTL_SRREQ;

		//Wait for self refresh acknowledge from the controller
		while((*pREG_DMC0_STAT&BITM_DMC_STAT_SRACK)==0);

	}

	#ifdef __ADSPSC58x__
	if(dmc_no==1)
	{
		//Workaround for the anomaly#20000067
		*pREG_DMC1_PHY_CTL1|=0x40000;

		//Wait for the DMC to be in IDLE state
		while((*pREG_DMC1_STAT&BITM_DMC_STAT_IDLE)==0);

		//Set the SRREQ bit to enter self refresh mode
		*pREG_DMC1_CTL|= BITM_DMC_CTL_SRREQ;

		//Wait for self refresh acknowledge from the controller
		while((*pREG_DMC1_STAT&BITM_DMC_STAT_SRACK)==0);

	}
	#endif
}

#pragma optimize_off
void DMC_Exit_Self_Refresh(uint32_t dmc_no)
{
	if(dmc_no==0)
	{
		//clear the SRREQ bit to exit self refresh mode
		*pREG_DMC0_CTL&=~BITM_DMC_CTL_SRREQ;

		//Wait for self refresh acknowledge from the controller
		while((*pREG_DMC0_STAT&BITM_DMC_STAT_SRACK)==1);

	}
	if(dmc_no==1)
	{
		//clear the SRREQ bit to enter self refresh mode
		*pREG_DMC1_CTL&=~BITM_DMC_CTL_SRREQ;

		//Wait for self refresh acknowledge from the controller
		while((*pREG_DMC1_STAT&BITM_DMC_STAT_SRACK)==1);

	}
}
